import os
from PIL import Image
import tempfile
import fileutils


def mb(size):
    return round(float(size) / (1024.0 * 1024.0), 2)


def run(parser):
    fileutils.ignore_folders.extend(['steam', 'cocoscenes'])
    fileutils.inspectResources()

    total_images_size = 0
    optimized = 0
    for i, file in enumerate(fileutils.images):
        fullpath = fileutils.root_dir + '/Resources/' + file
        size = fileutils.getSize(fullpath)
        total_images_size += size

        im = Image.open(fullpath)
        temppath = tempfile.gettempdir() + '/' + file
        fileutils.createDirForFile(temppath)
        im.save(temppath)

        new_size = fileutils.getSize(temppath)
        os.remove(temppath)

        diff = size - new_size
        if diff > int(parser.s):
            im.save(fullpath)
            optimized += diff
        print 'Processing {}% ({}/{})'.format(int(i * 100.0 / len(fileutils.images)), i, len(fileutils.images))

    print 'Optimized {}mb. \nSumary size before: {}mb. \nFinaly sumary size: {}mb'. \
        format(mb(optimized), mb(total_images_size), mb(total_images_size - optimized))


if __name__ == '__main__':
    run()





class Parser(object):
    """docstring for Parser"""

    def __init__(self, argv):
        super(Parser, self).__init__()
        self.command = 'help'
        self.args = {}
        self._parse(argv)
        self.root = self.get('-r', '--root', default='.')
        self.platform = self.get('-p', '--platform')
        self.configuration = self.get('-c', '--configuration')
        self.build_version = self.get('--build_version')
        self.upload = self.get('--upload', None, 'y')[0] == 'y'
        self.scheme = self.get('--scheme', None, 'Syndicate')
        self.s = self.get('-s', None, default='128000')

        self.ios_no_build_archive = self.get('--ios_disable_archive', None, 'n')[0] == 'y'
        self.ios_no_export_ipa = self.get('--ios_disable_export', None, 'n')[0] == 'y'
        self.ios_disable_upload_inapps = self.get('--ios_disable_upload_inapps', None, 'n')[0] == 'y'

        if self.command == 'help':
            self.help()
            exit(0)

    def _parse(self, argv):
        if len(argv) > 1:
            self.command = argv[1]
            if self.command in ['-h', '--help']:
                self.command = 'help'
        i = 2
        while i < len(argv):
            arg = argv[i]
            if arg.startswith('-') and i + 1 < len(argv):
                value = argv[i + 1]
                self._add_arg(arg, value)
                i += 2
            else:
                self._add_arg(arg)
                i += 1

    def _add_arg(self, name, value):
        self.args[name] = value

    def get(self, arg, syn=None, default=None):
        if arg in self.args:
            return self.args[arg]
        if syn is not None and syn in self.args:
            return self.args[arg]
        return default if default is not None else ''

    def help(self):
        print ''
        print 'using: $ stereo7 command -r path_to_project [arguments]'
        print ''
        print '  -r (--root) path_to_project (Optional, default is current directory)'
        print ''
        print 'commands:'
        print ''
        print '  help: show help'
        print '    -h: show hiden options'
        print ''
        print '  inapps: build inapps from google spreedsheet'
        print '    no arguments'
        print ''
        print '  validate: validate Resources, configs and other static'
        print '    no arguments'
        print ''
        print '  compress: remove all extra info from jpg, png images'
        print '    -s: minimal size for optimize (KB).'
        print ''
        print '  build: build and upload (optional)'
        print '    -p, --platform:           platform (ios, android, windows. In future: osx, steam)'
        print '    -c, --configuration:      configuration_name (debug, release. In future: pre-release)'
        print '    --build_version:          version (Optional, number of build version)'
        print '    --upload version:         Optional, use only on ios platform. (y, yes, n, no)'
        print '    --scheme scheme:          Optional, use only on ios platform. (name of scheme to build. Default is "Syndicate")'
        print ''
        if self.get('-h', None, 'n')[0] == 'y':
            print ''
            print 'Hiden options'
            print '  build: build and upload (optional)'
            print '    --ios_disable_archive:             disable building archive'
            print '    --ios_disable_export:              disable export archive'
            print '    --ios_disable_upload_inapps:       disable uploading inapps'



import os
import xml.etree.ElementTree as ET
import fileutils
from project import Project
import build_constants

ConfigurationDebug = 0
ConfigurationRelease = 1
ConfigurationPreRelease = 2


def os_system(cmd, message_on_error=''):
    print 'run:', cmd
    result = os.system(cmd)
    print 'result', result
    if result != 0:
        if message_on_error:
            print message_on_error
        exit(-1)
    return True


def set_values(string, configuration='', scheme=''):
    string = string.replace('@{ROOT}', fileutils.root_dir)
    string = string.replace('@{APPLE_APP_ID}', Project.instance.apple_id)
    string = string.replace('@{APPLE_USER}', Project.instance.apple_auth_user)
    string = string.replace('@{APPLE_PASSWORD}', Project.instance.apple_auth_password)
    string = string.replace('@{APPLE_TEAM_ID}', Project.instance.apple_team_id)
    string = string.replace('@{CONFIGURATION}', configuration)
    string = string.replace('@{SCHEME}', scheme)
    return string


class PlatformWindow:

    def get_configuration_name(self, configuration):
        strs = ['GameDebug', 'Release', 'GameRelease']
        return strs[configuration]

    def set_version(self, packagename, app_version, build_version):
        return True

    def build(self, configuration):
        cmd = set_values(build_constants.win_cmd_build,
                         configuration=self.get_configuration_name(configuration)
                         )
        return os_system(cmd)


class PlatformAndroid:

    def build(self, configuration):
        tasks = ['assembleDebug', 'assembleRelease', 'assembleRelease']
        cmd = set_values(build_constants.android_cmd_build,
                         configuration=tasks[configuration]
                         )
        return os_system(cmd)

    def set_version(self, packagename, app_version, build_version):
        path = fileutils.root_dir + '/proj.android/app/AndroidManifest.xml'
        ET.register_namespace('android', 'http://schemas.android.com/apk/res/android')
        try:
            handle = open(path, 'r')
            tree = ET.parse(handle)
            version = '{}.{}'.format(app_version, build_version) if build_version else app_version
            if build_version and str(build_version) != '0':
                tree.getroot().attrib["{http://schemas.android.com/apk/res/android}versionCode"] = str(build_version)
            tree.getroot().attrib["{http://schemas.android.com/apk/res/android}versionName"] = version
            tree.getroot().attrib["package"] = packagename
            tree.write(path, encoding='utf-8', xml_declaration=True)
            return True
        except:
            print 'cannot open file', path
            exit(-1)


class PlatformIos:

    def build(self, configuration):
        if configuration == ConfigurationDebug or configuration == ConfigurationPreRelease:
            print 'ios build not supported Debug configuration. Use one [release]'
            return False

        scheme = self.arg_parser.scheme
        build = set_values(build_constants.ios_cmd_build, scheme=scheme)
        export_create = set_values(build_constants.ios_export_plist)
        export = set_values(build_constants.ios_upload_shell)
        upload = set_values(build_constants.ios_upload_shell)
        upload_inapps = set_values(build_constants.ios_cmd_upload_inapps)

        if not self.arg_parser.ios_no_build_archive:
            os_system(build)
        else:
            print 'Missing build archive'

        if not self.arg_parser.ios_no_export_ipa:
            os_system(export_create)
            os_system(export)
        else:
            print 'Missing export ipa'

        if self.arg_parser.upload:
            os_system(upload)
            if not self.arg_parser.ios_disable_upload_inapps:
                os_system(upload_inapps)
        else:
            print 'Missing upload build and inapps to iTunesConnect'

        return True

    def set_version(self, packagename, app_version, build_version):
        sh = '/usr/libexec/Plistbuddy'
        plist = '{}/proj.ios/ios/Info.plist'.format(fileutils.root_dir)
        version = '{}.{}'.format(app_version, build_version) if build_version else app_version
        os_system('{} -c "Set CFBundleShortVersionString {}" "{}"'.format(sh, version, plist))
        os_system('{} -c "Set CFBundleIdentifier {}" "{}"'.format(sh, packagename, plist))
        if build_version and str(build_version) != '0':
            os_system('{} -c "Set CFBundleVersion {}" "{}"'.format(sh, build_version, plist))
        return True


def run(package_name, app_version, arg_parser):
    configuration = arg_parser.configuration
    platform = arg_parser.platform
    build_version = arg_parser.build_version

    platforms = {'windows': PlatformWindow, 'android': PlatformAndroid, 'ios': PlatformIos}
    configurations = {'debug': ConfigurationDebug, 'release': ConfigurationRelease, 'pre-release': ConfigurationPreRelease, }

    build = platforms[platform]()
    build.arg_parser = arg_parser
    configuration = configurations[configuration]

    result = build.set_version(package_name, app_version, build_version) and \
        build.build(configuration)
    if not result:
        print 'Some error'
        exit(-1)



import fileutils


class _Cache:
    _units = []


def get_units_list():
    if _Cache._units:
        return _Cache._units
    units = []
    all_files = []
    all_files.extend(fileutils.getFilesList(fileutils.root_dir + '/Resources/ini/units/creep/', False))
    all_files.extend(fileutils.getFilesList(fileutils.root_dir + '/Resources/ini/units/tower/', False))
    all_files.extend(fileutils.getFilesList(fileutils.root_dir + '/Resources/ini/units/hero/', False))
    all_files.extend(fileutils.getFilesList(fileutils.root_dir + '/Resources/ini/units/', False))
    for file in all_files:
        if file.startswith('_'):
            continue
        if not file.endswith('.xml'):
            continue
        name = file[0: -4]
        if name in ['death', 'death2']:
            continue
        units.append(name)
    _Cache._units = units
    return units



import json
import fileutils
import os


class Project(object):

    instance = None

    def __init__(self, empty=False):
        super(Project, self).__init__()
        if not empty:
            self._parse()

    def create(self):
        data = {}
        data['app_package'] = ''
        data['app_name'] = ''
        data['app_version'] = ''
        data['google_spreedsheet_inapps_id'] = ''
        data['google_api_secret_path'] = ''
        data['apple_auth_user'] = ''
        data['apple_auth_password'] = ''
        data['apple_id'] = ''
        data['apple_team_id'] = ''

        path = fileutils.root_dir + '/project.json'
        open(path, 'w').write(json.dumps(data, sort_keys=True, indent=4))
        fileutils.createDir(fileutils.root_dir + '/store')
        fileutils.createDir(fileutils.root_dir + '/store/inapps.itmsp')
        fileutils.write(fileutils.root_dir + '/store/inapps.itmsp/machine-local-data.xml', '<root/>')
        fileutils.write(fileutils.root_dir + '/store/inapps.itmsp/metadata.xml', '<root/>')
        fileutils.write(fileutils.root_dir + '/store/android_inapps.csv',
                        'Product ID,Published State,Purchase Type,Auto Translate,Locale; Title; Description,Auto Fill Prices,Price,Pricing Template ID')

    def _parse(self):
        path = fileutils.root_dir + '/project.json'
        if not os.path.isfile(path):
            print 'Cannot find project file [project.json]'
            exit(-1)
        data_file = open(path)
        data = json.load(data_file)
        self.package = data['app_package']
        self.name = data['app_name']
        self.version = data['app_version']
        self.gg_inapps = data['google_spreedsheet_inapps_id']
        self.gg_secret_file = fileutils.root_dir + '/' + data['google_api_secret_path']
        self.apple_auth_user = data['apple_auth_user']
        self.apple_auth_password = data['apple_auth_password']
        self.apple_id = data['apple_id']
        self.apple_team_id = data['apple_team_id']
